# coding=utf-8
from unittest import TestCase
import os
from tempfile import mktemp

import dummydata


from earthdiagnostics.diagnostic import DiagnosticVariableOption
from earthdiagnostics.box import Box
from earthdiagnostics.general.rewrite import Rewrite
from mock import Mock, patch

from earthdiagnostics.modelingrealm import ModelingRealms


class TestRewrite(TestCase):

    def setUp(self):
        self.data_manager = Mock()

        self.diags = Mock()
        self.diags.model_version = 'model_version'
        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))

        self.box = Box()
        self.box.min_depth = 0
        self.box.max_depth = 100

        self.diag = Rewrite(self.data_manager, '20000101', 1, 1, ModelingRealms.atmos, 'ta', 'grid')
        self.var_file = mktemp('.nc')

    def tearDown(self):
        if os.path.exists(self.var_file):
            os.remove(self.var_file)

    def fake_parse(self, value):
        return value

    @patch.object(DiagnosticVariableOption, 'parse', fake_parse)
    def test_generate_jobs(self):

        jobs = Rewrite.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], Rewrite(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var', 'original'))
        self.assertEqual(jobs[1], Rewrite(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var', 'original'))

        jobs = Rewrite.generate_jobs(self.diags, ['diagnostic', 'ocean', 'var', 'grid'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], Rewrite(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var', 'grid'))
        self.assertEqual(jobs[1], Rewrite(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var', 'grid'))

        with self.assertRaises(Exception):
            Rewrite.generate_jobs(self.diags, ['diagnostic'])

        with self.assertRaises(Exception):
            Rewrite.generate_jobs(self.diags, ['diagnostic', '0', '0', '0', '0', '0', '0', '0'])

    def test_str(self):
        self.assertEqual(str(self.diag),
                         'Rewrite output Startdate: 20000101 Member: 1 Chunk: 1 Variable: atmos:ta Grid: grid')

    def test_compute(self):
        dummydata.model2.Model2(oname=self.var_file, var='ta', start_year=2000, stop_year=2000, method='constant',
                                constant=1)

        self.diag.variable_file = Mock()
        self.diag.variable_file.local_file = self.var_file
        self.diag.corrected = Mock()
        self.diag.compute()
        self.diag.corrected.set_local_file.assert_called_once()
